#include "p_signal.h"

/* functions */

EIF_INTEGER posix_kill(EIF_INTEGER pid, EIF_INTEGER sig)
{
  return kill(pid, sig);
}

EIF_INTEGER posix_sigaction(int sig, EIF_POINTER act, EIF_POINTER oact)
{
  return sigaction(sig, act, oact);
}


/* constants */

EIF_INTEGER const_sa_nocldstop()
{
  return SA_NOCLDSTOP;
}


EIF_INTEGER const_sigalrm()
{
  return SIGALRM;
}

EIF_INTEGER const_sighup()
{
  return SIGHUP;
}

EIF_INTEGER const_sigkill()
{
  return SIGKILL;
}

EIF_INTEGER const_sigpipe()
{
  return SIGPIPE;
}

EIF_INTEGER const_sigquit()
{
  return SIGQUIT;
}

EIF_INTEGER const_sigchld()
{
  return SIGCHLD;
}

EIF_INTEGER const_sigcont()
{
  return SIGCONT;
}

EIF_INTEGER const_sigstop()
{
  return SIGSTOP;
}

EIF_INTEGER const_sigtstp()
{
  return SIGTSTP;
}

EIF_INTEGER const_sigttin()
{
  return SIGTTIN;
}

EIF_INTEGER const_sigttou()
{
  return SIGTTOU;
}



/* struct sigaction */

EIF_INTEGER posix_sigaction_size()
{
  return (sizeof (struct sigaction));
}


EIF_POINTER posix_sa_handler(struct sigaction *act)
{
  return (act->sa_handler);
}

EIF_POINTER posix_sa_mask(struct sigaction *act)
{
  return (&(act->sa_mask));
}

EIF_INTEGER posix_sa_flags(struct sigaction *act)
{
  return (act->sa_flags);
}


void posix_set_sa_handler(struct sigaction *act, EIF_POINTER handler)
{
  act->sa_handler = handler;
}

void posix_set_sa_flags(struct sigaction *act, EIF_INTEGER flags)
{
  act->sa_flags = flags;
}


/* struct sigevent */

EIF_INTEGER posix_sigevent_size()
{
   return (sizeof (struct sigevent));
}

EIF_INTEGER posix_sigevent_sigev_notify(struct sigevent *p)
{
  return p->sigev_notify;
}

EIF_INTEGER posix_sigevent_sigev_signo(struct sigevent *p)
{
  return p->sigev_signo;
}


void posix_set_sigevent_sigev_notify(struct sigevent *p, EIF_INTEGER sigev_notify)
{
  p->sigev_notify = sigev_notify;
}

void posix_set_sigevent_sigev_signo(struct sigevent *p, EIF_INTEGER sigev_signo)
{
  p->sigev_signo = sigev_signo;
}


/* notify values */

EIF_INTEGER const_sigev_none()
{
  return SIGEV_NONE;
}

EIF_INTEGER const_sigev_signal()
{
  return SIGEV_SIGNAL;
}

EIF_INTEGER const_sigev_thread()
{
#ifdef SIGEV_THREAD
  return SIGEV_THREAD;
#else
  return 0;
#endif
}

