indexing

   description: "Class that covers Posix file descriptor routines."

   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 13:36:48 +0200 $"
   revision: "$Revision: 1.9 $"
   

class 

   POSIX_FILE_DESCRIPTOR

inherit

   BASE_FILE_DESCRIPTOR
      redefine
         close
      end

   POSIX_SYSTEM

   
creation

   open,
   open_read,
   open_write,
   open_read_write,
   open_truncate,
   create_read_write,
   create_with_mode,
   make_as_duplicate,
   make_from_file,
   attach_to_fd


feature -- close
   
   close is
      do
         precursor
         my_termios := Void
      end

   close_on_execute is
         -- close this descriptor when forking
      local
         flags: INTEGER
      do
         flags := posix_fcntl (value, F_GETFD)
         flags := flip_bits (flags, FD_CLOEXEC, False)
         safe_call (posix_fcntl_arg (value, F_SETFD, flags))
      end

   
feature -- synchronisation
   
   synchronize, fsync is
      -- synchronize the state of a file (includes synchronize_data)
      require
         synchronize_valid: supports_file_synchronization
      do
         safe_call (posix_fsync (value))
      end
   
   synchronize_data, fdatasync is
      -- synchronize the data of a file
      require
         synchronize_valid: supports_synchronized_io
      do
         safe_call (posix_fdatasync (value))
      end
   
   
feature -- locking

   get_lock (a_lock: POSIX_LOCK): BOOLEAN is
         -- gets lock information, returns True if a lock is set on
         -- the region in a_lock. a_lock is overwritten with that lock
      do
         safe_call (posix_fcntl_lock (value, F_GETLK, a_lock.buf.ptr))
         Result := not a_lock.allow_all
      end
   
   set_lock_failed: BOOLEAN
         -- Test after set_lock if lock did success
   
   attempt_lock (a_lock: POSIX_LOCK) is
         -- attempt to set lock, if not possible, set
         -- `set_lock_failed'
      local
         r: INTEGER
      do
         r := posix_fcntl_lock (value, F_SETLK, a_lock.buf.ptr)
         set_lock_failed := r = EAGAIN
         if r = -1 then
            if not set_lock_failed then
               raise_posix_error
            end
         end
      end

   set_lock (a_lock: POSIX_LOCK) is
         -- attempt to set lock, wait if necessary
      do
         safe_call (posix_fcntl_lock (value, F_SETLKW, a_lock.buf.ptr))
      end
   
   
feature -- queries
   
   terminal: POSIX_TERMIOS is
         -- terminal settings
      require
         valid_file_descriptor: is_attached_to_terminal
      do
         if my_termios = Void then
            create my_termios.make (Current)
         end
         Result := my_termios
      ensure
         valid_result: Result /= Void
      end

   ttyname: STRING is
         -- Terminal path name, or empty if this file descriptor does
         -- not refer to a terminal
      do
         Result := sh.pointer_to_string (posix_ttyname (value))
      end
   

feature {NONE} -- private state
   
   my_termios: POSIX_TERMIOS
         -- lazy build access to terminal setttings
   
  
end -- class POSIX_FILE_DESCRIPTOR
