indexing

   description: "Abstract fork root class."
   
   usage: "Implement execute method and:%
   %- pass this class to your fork method if you inherit from%
   %  POSIX_CURRENT_PROCESS.fork%
   %- call the fork method and pass yourself."
   
   
   author: "Berend de Boer"
   date: "$Date: Sat, 14 Oct 2000 08:59:36 +0200 $"
   revision: "$Revision: 1.3.1.4 $"
   

deferred class 

   POSIX_FORK_ROOT
   
   
inherit
   
   POSIX_CHILD_PROCESS
      rename
         is_pid_valid as is_child_pid_valid
      redefine
         pid
      select
         pid
      end

   POSIX_CURRENT_PROCESS
      redefine
         pid
      select
         is_pid_valid
      end
   
   CAPI_STDLIB

   
feature -- process properties

   pid: INTEGER is
         -- either the current process identifier or the child's
      do
         if my_pid > 0 then
            Result := my_pid
         else
            Result := posix_getpid
         end
      end

   is_valid_child_process: BOOLEAN is
         -- returns True if this object seems to refer to a valid child process
         -- the real child process might have stopped though
      do
         Result := my_pid > 0
      end
   

feature {POSIX_CURRENT_PROCESS}
   
--     set_child_pid (a_child_pid: INTEGER) is
--           -- initialize pid as seen by parent
--        do
--           child_pid := a_child_pid
--        end
   
   frozen start is
         -- called by the `fork' routine
         --makes sure child does not return
      do
         terminated := False
         execute
         posix_exit (0)
      rescue
         -- ????
         posix_exit (0)
      end
   
   
feature -- deferred routines
   
   execute is         
      -- start if child process
      deferred
      end
   
   
feature -- accessible state
   
--     child_pid: INTEGER
--           -- pid of the child, not valid for forked child itself

--     is_child: BOOLEAN is
--           -- a forked class has two sides: one is seen by the parent, the
--           -- other by the child
--        do
--           Result := child_pid = 0
--        ensure
--           Result implies not is_parent
--        end   
   
--     is_parent: BOOLEAN is
--           -- a forked class has two sides: one is seen by the parent, the
--           -- other by the child
--        do
--           Result := child_pid > 0
--        ensure
--           Result implies not is_child
--        end


feature -- termination info
   
   is_terminated_normally,
   is_exited: BOOLEAN is
         -- has this process been terminated normally
      require
         valid_status_info: is_terminated
      do
         Result := posix_wifexited (termination_info)
      end

   exit_code: INTEGER is
      -- low-order 8 bits of call to _exit or exit for this process
      require else
         terminated_normally: is_terminated_normally
      do
         Result := posix_wexitstatus (termination_info)
      end

   is_signaled: BOOLEAN is
         -- child process was terminated due to receipt of a signal 
         -- that was not caught
      require
         valid_status_info: is_terminated
      do
         Result := posix_wifsignaled (termination_info)
      end

   signal_code: INTEGER is
         -- signal of process terminated abnormally or was stopped
      require
         valid_status_info: is_terminated
         terminated_by_signal: is_signaled
      do
         Result := posix_wtermsig (termination_info)
      end


end -- class POSIX_FORK_ROOT
