indexing

   description: "Class that covers a single POSIX signal. You can%
   %query and determine how a signal is handled for the current process."

   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 13:36:48 +0200 $"
   revision: "$Revision: 1.4 $"

class 

   POSIX_SIGNAL
   
inherit

   POSIX_BASE

   PAPI_SIGNAL

creation
   
   make
   

feature -- creation

   make (a_value: INTEGER) is
      do
         value := a_value
         create sigaction.allocate (posix_sigaction_size)
         refresh
      end
   

feature -- set signal properties, make effective with apply
   
   apply is
         -- make changes effective
      do
         safe_call (posix_sigaction (value, sigaction.ptr, default_pointer))
      end   
   
   set_child_stop (stop: BOOLEAN) is
         -- generate SIGCHLD when children stop
      local
         flags: INTEGER
      do
         flags := 
            flip_bits (posix_sa_flags (sigaction.ptr), 
                       SA_NOCLDSTOP, 
                       not child_stop)
         posix_set_sa_flags (sigaction.ptr, flags)
      end

   set_default_action is
         -- install signal-specific default action
      do
         posix_set_sa_handler (sigaction.ptr, SIG_DFL)
      end

   set_ignore_action is
         -- ignore signal
      require
         not_sig_child: value /= SIGCHLD
      do
         posix_set_sa_handler (sigaction.ptr, SIG_IGN)
      end
   
   set_mask (a_mask: POSIX_SIGNAL_SET) is
      do
      end


feature -- signal state

   child_stop: BOOLEAN is
         -- generate SIGCHLD when children stop
      do
         Result := 
            posix_and (posix_sa_flags (sigaction.ptr), SA_NOCLDSTOP) = 1
      end

   is_defaulted: BOOLEAN is
         -- signal is handled by its specific default action
      do
         Result := posix_sa_handler (sigaction.ptr) = SIG_DFL
      end
  
   handler: POINTER is
         -- pointer to function which catches this signal
      do
         Result := posix_sa_handler (sigaction.ptr)
      end

   is_ignored: BOOLEAN is
         -- signal is ignored
      do
         Result := posix_sa_handler (sigaction.ptr) = SIG_IGN
      end

   mask: POSIX_SIGNAL_SET
   
   value: INTEGER
         -- the signal

   refresh is
         -- get latest state for this signal   
      do
         safe_call (posix_sigaction (value, default_pointer, sigaction.ptr))
      end


feature {NONE} -- private state
   
   sigaction: STDC_DYNAMIC_MEMORY
         -- C sigaction struct   


invariant   
   
   has_memory: sigaction /= Void


end -- class POSIX_SIGNAL
